import json
import logging
from functools import cache
from urllib.request import urlopen

import jwt
from django.conf import settings

logger = logging.getLogger(__name__)


@cache
def get_public_keys():
    # Update this to 'https://{clerk_frontend_api}/.well-known/jwks.json'
    # Note: The content of this endpoint will never change, so it should
    # be cached on the server instead of requested with each API call
    url = settings.CLERK_JWT_JWKS_URL
    response = urlopen(url)
    data_json = json.loads(response.read())
    logger.debug(f"jwks url response: {data_json}")
    logger.debug(f"JWT Keys: {data_json['keys']}")
    return data_json['keys']


def validate(token: str):
    try:
        data = decode(token)
        logger.debug(f"Data {data}")
    except jwt.exceptions.PyJWTError as e:
        logger.error(f"JWT decoding error: {e}")
        return False

    return bool(data)


def decode(token: str):
    logger.debug(f"Decoding token {token}")
    return jwt.decode(
        token,
        key=settings.CLERK_JWT_PEM_PUBLIC_KEY,
        algorithms=["RS256"],
    )



from django_clerk.models import RawClerkEvent
from django.http import HttpResponse
from django.conf import settings

from django.views.decorators.csrf import csrf_exempt
from svix.webhooks import Webhook, WebhookVerificationError

import logging

logger = logging.getLogger(__name__)


@csrf_exempt
def clerk_webhook_handler(request):
    logger.info("Received Clerk Webhook")
    headers = request.headers
    payload = request.body

    try:
        wh = Webhook(settings.CLERK_WEBHOOK_SECRET)
        msg = wh.verify(payload, headers)
    except WebhookVerificationError as e:
        return HttpResponse(status=400)

    RawClerkEvent.objects.create(json_blob=msg)

    return HttpResponse(status=204)



import logging
from django.contrib.auth import get_user_model

from .jwt import decode, validate

User = get_user_model()
logger = logging.getLogger(__name__)


class ClerkMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        request = self.process_request(request)
        response = self.get_response(request)
        request = self.process_response(request, response)

        return response

    def process_request(self, request):
        token = self.get_auth_token(request)
        if token and self.validate_token(token):
            request.user = self.get_user_from_token(token)
            logger.debug(f"Request User: {request.user}")
        else:
            request.user = None
        return request

    def process_response(self, request, response):
        return response

    def get_auth_token(self, request):
        return request.headers.get("Authorization", "").split("Bearer ")[-1]

    def validate_token(self, token):
        return validate(token)

    def get_user_from_token(self, token):
        """
        Retrieves a user object based on the provided token.
        This method can be overridden to customize the user 
        retrieval process.

        Args:
            token (str): The token used to identify the user.

        Returns:
            User: The user object associated with the token. If
            the user does not exist, a new user object will be created.
        """
        data = decode(token)
        if User.objects.filter(email=data["email"]).exists():
            user = User.objects.get(email=data["email"])
        else:
            user = User.objects.create(email=data["email"])

        return user
